import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import Core
import Check
import I18n

import "../core"
import "../widgets"

ModalForm {
    id: root
    objectName: "addPositionMultipleChoiceForm"
    Style.class: "addPositionMultipleChoiceForm"

    implicitWidth: Style.parameter("geometry.addPositionMultipleChoice.width", 1010)
    implicitHeight: Style.parameter("geometry.addPositionMultipleChoice.height", 1352)

    Context.inputSources: InputSources.Keyboard

    allowFloatKeyboard: true

    signal hideKeyboard()

    Connections {
        target: Qt.inputMethod
        function onVisibleChanged() {
            if (!Qt.inputMethod.visible)
                hideKeyboard()
        }
    }

    property string nameFilter: ""
    property string barcodeFilter: ""
    property string codeFilter: ""
    property string priceFilter: ""

    property var selectedItem: null

    function updateModel() { positionsModel.update(nameFilter, barcodeFilter, codeFilter, priceFilter, positionsModel.top) }

    property list<QtObject> methods: [
        QtObject {
            property alias filter: root.nameFilter
            property string methodTr: "addPositionMultipleChoiceNameFilter"
            property string placeholderTr: "addPositionMultipleChoiceNameFilterPlaceholder"
        },
        QtObject {
            property alias filter: root.barcodeFilter
            property string methodTr: "addPositionMultipleChoiceBarcodeFilter"
            property string placeholderTr: "addPositionMultipleChoiceBarcodeFilterPlaceholder"
        },
        QtObject {
            property alias filter: root.codeFilter
            property string methodTr: "addPositionMultipleChoiceCodeFilter"
            property string placeholderTr: "addPositionMultipleChoiceCodeFilterPlaceholder"
        },
        QtObject {
            property alias filter: root.priceFilter
            property string methodTr: "addPositionMultipleChoicePriceFilter"
            property string placeholderTr: "addPositionMultipleChoicePriceFilterPlaceholder"
        }
    ]

    property int methodIndex: 0
    property var currentMethod: methods[methodIndex]

    onActivated: Qt.callLater(() => { search.forceActiveFocus() })

    contentItem: ColumnLayout {
        objectName: "mainLayout"
        Layout.fillWidth: true
        Layout.fillHeight: true
        spacing: 30

        ColumnLayout {
            spacing: 16

            SGroupBox {
                id: pickListSearch
                objectName: "searchBox"
                Style.class: "addPositionMultipleChoiceSearch"
                Layout.fillWidth: true
                topPadding: 0
                bottomPadding: 0

                contentItem: STextEdit {
                    id: search
                    objectName: "searchField"
                    Style.class: "addPositionMultipleChoiceSearch"
                    clip: true
                    horizontalAlignment: STextEdit.AlignHCenter
                    verticalAlignment: STextEdit.AlignVCenter
                    focus: Qt.inputMethod.visible
                    text: currentMethod.filter.length ? currentMethod.filter : ""
                    topPadding: 30
                    bottomPadding: 30
                    rightPadding: 4 * clearFilterButton.width / 3
                    leftPadding: 4 * clearFilterButton.width / 3

                    Timer {
                        id: filterTimer
                        repeat: false
                        interval: 500

                        onTriggered: () => {
                            currentMethod.filter = search.text
                            positionsModel.top = 0
                            root.updateModel()
                        }
                    }

                    SLabel {
                        Style.class: "addPositionMultipleChoiceSearchPlaceholder"
                        Tr { code: currentMethod.placeholderTr }
                        anchors.centerIn: parent
                        visible: search.text.length === 0
                    }

                    ActionButton {
                        id: clearFilterButton
                        Style.class: "addPositionMultipleChoiceClearFilter"
                        objectName: "clearFilterButton"
                        icon: Style.parameter("icon.addPositionMultipleChoice.clear")
                        iconSize: 37.5
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        opacity: search.text.length > 0 ? 1 : 0
                        onClicked: () => { currentMethod.filter = ""; filterTimer.restart() }
                        Behavior on opacity { NumberAnimation { duration: Style.animationDuration("transition.multipleChoice.filter", 250) } }
                    }

                    onDisplayTextChanged: () => { if (currentMethod.filter !== search.text) filterTimer.restart() }
                }
            }

            RowLayout {
                objectName: "inputMethodButtonsLayout"

                spacing: 16
                Layout.alignment: Qt.AlignCenter
                Layout.fillWidth: true
                Layout.preferredHeight: 56

                Repeater {
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    model: root.methods
                    delegate: ActionButton {
                        Style.class: "addPositionMultipleChoiceMethod"
                        Layout.preferredHeight: 56
                        rightPadding: 40
                        leftPadding: 40

                        enabled: root.methodIndex !== model.index
                        Tr { code: model.methodTr }

                        SvgImage {
                            anchors.topMargin: 8
                            anchors.rightMargin: 12
                            anchors.top: parent.top
                            anchors.right: parent.right
                            opacity: filter.length > 0
                            source: root.methodIndex !== model.index ? Style.parameter("icon.addPositionMultipleChoice.filter") : Style.parameter("icon.addPositionMultipleChoice.filter_selected")
                            width: 24
                            height: 24

                            Behavior on opacity { NumberAnimation { duration: Style.animationDuration("transition.multipleChoice.filter", 250) } }
                        }

                        onClicked: () => methodIndex = model.index
                    }
                }
            }
        }

        RowLayout {
            id: positionListLayout
            spacing: scrollBar.size < 0.99 ? 15 : 0

            Behavior on spacing {
                NumberAnimation { easing.type: Easing.InOutQuad; duration: Style.animationDuration("transition.multipleChoice.scrollBar", 250) }
            }

            ListView {
                id: list
                Layout.fillHeight: true
                Layout.fillWidth: true

                ScrollBar.vertical: scrollBar

                onCurrentIndexChanged: {
                    if (!root.selectedItem)
                        return

                    // чтобы сбрасывать активный элемент, когда он пропадает из поля зрения
                    const crossingTop = root.selectedItem.y < list.contentY
                    const crossingBottom = root.selectedItem.y + root.selectedItem.height > list.contentY + list.height
                    const fullVisible = !crossingTop && !crossingBottom
                    if (!fullVisible)
                        root.selectedItem = null
                }

                model: PositionMultipleChoiceModel {
                    id: positionsModel
                    property int top: 0
                    context: Context.context
                    onChanged: () => {
                        root.selectedItem = null
                        if (top === 0)
                            return
                        list.positionViewAtIndex(maxCount / 2, ListView.Center)
                    }
                }

                clip: true
                headerPositioning: ListView.PullBackHeader

                header: SGroupBox {
                    objectName: "listViewHeader"
                    Style.class: "addPositionMultipleChoiceListHeader"
                    id: listHeader
                    width: list.width
                    padding: 20
                    topPadding: 10
                    z: parent.z + 3 // чтобы отображать поверх позиций

                    onClicked: { list.forceActiveFocus(Qt.MouseFocusReason) }

                    contentItem: RowLayout {
                        spacing: 30
                        SLabel {
                            objectName: "listHeaderName"
                            Style.class: "addPositionMultipleChoiceListHeader"
                            Tr { code: "addPositionMultipleChoiceNameHeader" }
                            Layout.fillWidth: true
                            Layout.alignment: Qt.AlignLeft
                            horizontalAlignment: Qt.AlignLeft
                        }
                        SLabel {
                            objectName: "listHeaderBarcode"
                            Style.class: "addPositionMultipleChoiceListHeader"
                            Tr { code: "addPositionMultipleChoiceBarcodeHeader" }
                            elide: SLabel.ElideRight
                            Layout.alignment: Qt.AlignLeft
                            horizontalAlignment: Qt.AlignLeft
                            Layout.minimumWidth: 200
                            Layout.maximumWidth: 200
                        }
                        SLabel {
                            objectName: "listHeaderCode"
                            Style.class: "addPositionMultipleChoiceListHeader"
                            Tr { code: "addPositionMultipleChoiceCodeHeader" }
                            elide: SLabel.ElideRight
                            Layout.alignment: Qt.AlignLeft
                            horizontalAlignment: Qt.AlignLeft
                            Layout.minimumWidth: 87
                            Layout.maximumWidth: 87
                        }
                        SLabel {
                            objectName: "listHeaderPrice"
                            Style.class: "addPositionMultipleChoiceListHeader"
                            Tr { code: "addPositionMultipleChoicePriceHeader" }
                            elide: SLabel.ElideRight
                            Layout.alignment: Qt.AlignRight
                            horizontalAlignment: Qt.AlignRight

                            Layout.minimumWidth: 125
                            Layout.maximumWidth: 125
                        }
                    }
                }

                delegate: SGroupBox {
                    id: delegateBox
                    Style.class: "addPositionMultipleChoicePosition"
                    width: list.width
                    padding: 0
                    leftPadding: 20
                    rightPadding: 20
                    height: Style.parameter("geometry.addPositionMultipleChoice.positionDelegate.height", 92)

                    property bool selected: root.selectedItem === this
                    property string barcode: model.barcode

                    onClicked: {
                        list.forceActiveFocus(Qt.MouseFocusReason)
                        root.selectedItem = this
                    }

                    background: Rectangle {
                        color: {
                            if (parent.selected)
                                return root.Style.color("green")
                            return (index + 1) % 2 == 0 ? root.Style.color("gray") : root.Style.color("white")
                        }
                        radius: parent.Style.border.radius
                        opacity: {
                            if (parent.selected)
                                return 0.25
                            return (index + 1) % 2 == 0 ? 0.2 : 1
                        }
                        border.color: parent.Style.border.color
                        border.width: parent.Style.border.width
                    }

                    contentItem: Item {
                        id: position

                        RowLayout {
                            anchors.fill: parent
                            spacing: 30

                            SLabel {
                                Style.class: "addPositionMultipleChoicePositionInfo"
                                text: name
                                elide: SLabel.ElideRight
                                wrapMode: Text.WordWrap
                                maximumLineCount: 2
                                Layout.fillWidth: true
                                Layout.alignment: Qt.AlignLeft
                            }
                            SLabel {
                                Style.class: "addPositionMultipleChoicePositionInfo"
                                text: barcode
                                elide: SLabel.ElideRight
                                Layout.alignment: Qt.AlignLeft
                                Layout.minimumWidth: 200
                                Layout.maximumWidth: 200
                            }
                            SLabel {
                                Style.class: "addPositionMultipleChoicePositionInfo"
                                text: code
                                elide: SLabel.ElideRight
                                Layout.alignment: Qt.AlignLeft
                                Layout.minimumWidth: 87
                                Layout.maximumWidth: 87
                            }
                            SLabel {
                                Style.class: "addPositionMultipleChoicePositionInfo"
                                text: Money.format(Money.create(parseFloat(price).toFixed(2) * 100))

                                Layout.alignment: Qt.AlignLeft
                                horizontalAlignment: Qt.AlignRight
                                Layout.minimumWidth: 125
                                Layout.maximumWidth: 125
                            }
                        }
                    }
                }

                // Дозагрузка данных при скролинге
                onMovementEnded: {
                    if (atYBeginning) {
                        if (model.top === 0)
                            return
                        positionsModel.top -= model.maxCount / 2
                        root.updateModel()
                    }
                    if (atYEnd) {
                        if (model.count < model.maxCount)
                            return
                        positionsModel.top += model.maxCount / 2
                        root.updateModel()
                    }
                }
            }

            ScrollBar {
                id: scrollBar
                policy: ScrollBar.AlwaysOn
                padding: 0

                background: Rectangle {
                    color: Style.color("graySoft")
                    radius: width / 2
                }

                contentItem: Rectangle {
                    color: Style.color("green")
                    radius: width / 2
                }

                Behavior on Layout.preferredWidth {
                    NumberAnimation { easing.type: Easing.InOutQuad; duration: Style.animationDuration("transition.multipleChoice.scrollBar", 250) }
                }

                function getWidth() {
                    if (!list.contentHeight || !list.height)
                        return 0
                    return list.contentHeight > list.height && size < 0.99 ? 10 : 0
                }
                Layout.preferredWidth: getWidth()
                Layout.fillHeight: true
            }
        }

        GridLayout {
            id: buttonsLayout
            objectName: "controlButtonsLayout"
            Layout.fillWidth: true
            property bool vertical: true
            flow: vertical ? GridLayout.TopToBottom : GridLayout.LeftToRight
            layoutDirection: vertical ? Qt.LeftToRight : Qt.RightToLeft

            rowSpacing: 20
            columnSpacing: 20

            ActionButton {
                id: okButton
                objectName: "addButton"
                Style.class: "addPositionMultipleChoiceAdd"
                implicitWidth: 1

                Tr { code: "addPositionMultipleChoiceAdd" }

                Layout.fillWidth: true
                Layout.preferredHeight: 105

                Action {
                    id: inputAction
                    command: "CORE_INPUT"

                    property string data: root.selectedItem?.barcode?? ""
                    property string rawData: root.selectedItem?.barcode?? ""
                    property int dataSource: EInput.Ui
                }

                enabled: root.selectedItem !== null
                onClicked: { inputAction.exec() }
            }

            ActionButton {
                id: cancelButton
                objectName: "cancelButton"
                Style.class: "addPositionMultipleChoiceClose"
                icon: Style.parameter("icon.input.cancel")
                implicitWidth: 1

                Tr { code: "addPositionMultipleChoiceCancel" }

                Layout.fillWidth: true
                Layout.preferredHeight: 105

                onClicked: removeContext()
            }
        }
    }
}
